﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net.Http.Headers;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Http.Features;
using Microsoft.AspNetCore.Mvc;
using Microsoft.WindowsAzure.Storage;
using Microsoft.WindowsAzure.Storage.Auth;
using Microsoft.WindowsAzure.Storage.Blob;
using Microsoft.WindowsAzure.Storage.RetryPolicies;

namespace svc.Controllers
{
    [Route("api/[controller]")]
    [ApiController]
    public class DefaultController : ControllerBase
    {
        private IWebHostEnvironment _hostingEnv;
        private static string newfileName = Guid.NewGuid().ToString();
        private const int MaxBlockSize = 6 * 1024 * 1024; // Approx. 6MB chunk size

        public DefaultController(IWebHostEnvironment hostEnvironment)
        {
            this._hostingEnv = hostEnvironment;
        }

        [HttpPost]
        [Route("Save")]
        public async Task Save(IList<IFormFile> chunkFile, IList<IFormFile> UploadFiles)
        {
            long size = 0;
            try
            {
                string blobName = "mytest1.zip";
                List<byte[]> b = new List<byte[]>();

                // for chunk-upload
                foreach (var file in chunkFile)
                {
                  
                    size += file.Length;
                    var ms = new MemoryStream();
                    file.OpenReadStream().CopyTo(ms);
                    byte[] Value = ms.ToArray();
                    b.Add(Value);
                }
                IEnumerable<byte> result = Enumerable.Empty<byte>();
                foreach (byte[] bytes in b)
                {
                    result = result.Concat(bytes);
                }
                byte[] newArray = result.ToArray();

                var storageCredentials = new StorageCredentials("<ACCOUNT NAME>", "<YOUR KEY>");
                var cloudStorageAccount = new CloudStorageAccount(storageCredentials, true);
                var cloudBlobClient = cloudStorageAccount.CreateCloudBlobClient();
                var container = cloudBlobClient.GetContainerReference("filo");
                await container.CreateIfNotExistsAsync();
                var newBlob = container.GetBlockBlobReference(blobName);
                newBlob.StreamWriteSizeInBytes = 6 * 1024 * 1024;
                // This sets the size of the blocks to use when you do a Put Blob and it breaks it into blocks to upload because the file is larger than the value of SingleBlobUploadThresholdInBytes.
                // By default, this is 4MB(4 * 1024 * 1024).

                newBlob.StreamMinimumReadSizeInBytes = 6 * 1024 * 1024;

                //set the blob upload timeout and retry strategy
                BlobRequestOptions options = new BlobRequestOptions();
                options.ServerTimeout = new TimeSpan(0, 180, 0);
                options.RetryPolicy = new ExponentialRetry(TimeSpan.Zero, 20);

                //get the file blocks of 2MB size each and perform upload of each block
                HashSet<string> blocklist = new HashSet<string>();
                List<FileBlock> bloksT = GetFileBlocks(newArray).ToList();
                foreach (FileBlock block in GetFileBlocks(newArray))
                {
                    await newBlob.PutBlockAsync(
                         block.Id,
                         new MemoryStream(block.Content, true), null,
                         null, options, null
                         );

                    blocklist.Add(block.Id);

                }
                //commit the blocks that are uploaded in above loop
                await newBlob.PutBlockListAsync(blocklist, null, options, null);

                Console.WriteLine("Done");
            }
            catch (Exception e)
            {
                // Response.Clear();
                Response.StatusCode = 204;
                Response.HttpContext.Features.Get<IHttpResponseFeature>().ReasonPhrase = "File failed to upload";
                Response.HttpContext.Features.Get<IHttpResponseFeature>().ReasonPhrase = e.Message;
            }

        }


        [HttpPost]
        [Route("Remove")]
        public async Task<IActionResult> Remove(IList<IFormFile> UploadFiles)
        {
            var storageCredentials = new StorageCredentials("testcenter", "Pq/k6cYwLGZJ0oQGXJU6a5OTqawfGyhUlSPbWmGemw3dUnK4L+kJREL3DrRsEsYsS9QfEiz6OwgHDG6bKzIhcA==");
            var cloudStorageAccount = new CloudStorageAccount(storageCredentials, true);
            var cloudBlobClient = cloudStorageAccount.CreateCloudBlobClient();
            var container = cloudBlobClient.GetContainerReference("filo");
            await container.CreateIfNotExistsAsync();

            try
            {
                foreach (var file in UploadFiles)
                {
                    var fileName = ContentDispositionHeaderValue.Parse(file.ContentDisposition).FileName.Trim('"');
                    //var filePath = Path.Combine(_hostingEnv.WebRootPath);
                    //var fileSavePath = filePath + @"\" + fileName;
                    //if (System.IO.File.Exists(fileSavePath))
                    //{
                    //    System.IO.File.Delete(fileSavePath);
                    //}
                    var newBlob = container.GetBlockBlobReference("myGame");
                    await newBlob.DeleteIfExistsAsync(); //DELETE A FILE

                }
            }
            catch (Exception e)
            {
                Response.Clear();
                Response.StatusCode = 200;
                Response.HttpContext.Features.Get<IHttpResponseFeature>().ReasonPhrase = "File removed successfully";
                Response.HttpContext.Features.Get<IHttpResponseFeature>().ReasonPhrase = e.Message;
            }

            return Content("Remove Action");

        }


        private static IEnumerable<FileBlock> GetFileBlocks(byte[] fileContent)
        {
            HashSet<FileBlock> hashSet = new HashSet<FileBlock>();
            if (fileContent.Length == 0)
                return new HashSet<FileBlock>();

            int blockId = 0;
            int ix = 0;

            int currentBlockSize = MaxBlockSize;

            while (currentBlockSize == MaxBlockSize)
            {
                if ((ix + currentBlockSize) > fileContent.Length)
                    currentBlockSize = fileContent.Length - ix;

                byte[] chunk = new byte[currentBlockSize];
                Array.Copy(fileContent, ix, chunk, 0, currentBlockSize);

                hashSet.Add(
                    new FileBlock()
                    {
                        Content = chunk,
                        Id = Convert.ToBase64String(System.BitConverter.GetBytes(blockId))
                    });

                ix += currentBlockSize;
                blockId++;
            }

            return hashSet;
        }

        
        internal class FileBlock
        {
            public string Id
            {
                get;
                set;
            }

            public byte[] Content
            {
                get;
                set;
            }
        }

    }

}